==> ToolChain_STM32_1.0.49.tar.bz: ToolChain_STM32_1.0.49/Template/examples/example_threading_semaphores.c <==
/*{ ExampleThreading::.c ************************************************
 
 Example code demonstrating how to use inter thread communication.
 
}*/

#include "example_threading_semaphores.h"

#define AmountProducers 1
#define AmountConsumers 6

//{ Function definitions *************************************************

void threading_init() {
#ifdef EXTENSION_400_lcd_320x240_olimex

    if (1) { // init_LCD Start
        RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2, ENABLE);

        GPIO_InitTypeDef GPIO_InitStructure;
        GPIO_InitStructure.GPIO_Pin = GPIO_Pin_All;
        GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
        GPIO_Init(GPIOA, &GPIO_InitStructure);
        GPIO_Init(GPIOB, &GPIO_InitStructure);
        GPIO_Init(GPIOC, &GPIO_InitStructure);
        GPIO_Init(GPIOD, &GPIO_InitStructure);
        GPIO_Init(GPIOE, &GPIO_InitStructure);
        GPIO_Init(GPIOF, &GPIO_InitStructure);
        GPIO_Init(GPIOG, &GPIO_InitStructure);

#ifdef RCC_AHBPeriph_FSMC
        /* Enable the FSMC Clock */
        RCC_AHBPeriphClockCmd(RCC_AHBPeriph_FSMC, ENABLE);
#endif

        LCD_Init();
    }

    LCD_SetBacklight(1);
    u16_t BgColor = LCD_RGB_TO_LCDCOLOR(200,255,200);
    LCD_Clear(BgColor);

    //LCD_DrawChar(1,1, '>', LCD_COLOR_YELLOW, LCD_COLOR_BLACK);
    LCD_ConSetColor(BgColor, LCD_COLOR_BLACK);
#endif

}
void threading_start() {
    threading_init();

    static ets_ThreadInfo_t ThreadInfo; // must be static to survive end of this function!

    // Mutex being used to wait until task has copied its ThreadInfo
    ThreadInfo.Lock_Init = ttc_mutex_create();
    Assert(NULL != ThreadInfo.Lock_Init, ec_Malloc);

    // create counting semaphore Producer -> Consumer
    ThreadInfo.SemaphoreProducer2Consumer = ttc_semaphore_create(30, 0);
    Assert(NULL != ThreadInfo.SemaphoreProducer2Consumer, ec_Malloc);

    // create queue Producer/Consumer -> Print
    ThreadInfo.QueuePrint = ttc_queue_create(5, sizeof(et_PrintJob_t));
    Assert(NULL != ThreadInfo.QueuePrint, ec_Malloc);
    
    ttc_mutex_lock(ThreadInfo.Lock_Init, 0);  // lock structure
    Assert(0 != ttc_task_create( taskPrint,                             //{ function to start as thread
                                 "Print",          // thread name (just for debugging)
                                 4 * TTC_TASK_MINIMAL_STACK_SIZE, // stack size
                                 (void *) &ThreadInfo,                  // passed as argument to taskValue()
                                 1,            // task priority (higher values mean more process time)
                                 (xTaskHandle*) NULL                    // can return a handle to created task
                                 ),
           ec_Malloc); //}
    while (ttc_mutex_lock(ThreadInfo.Lock_Init, -1) != tqe_OK);  // wait until task has released structure

    ThreadInfo.DisplayRow = 5;

    for (int Amount = 0; Amount < AmountProducers; Amount++) {
        Assert(0 != ttc_task_create( taskProducer,                          //{ function to start as thread
                                     "Producer",       // thread name (just for debugging)
                                     4 * TTC_TASK_MINIMAL_STACK_SIZE, // stack size
                                     (void *) &ThreadInfo,                  // passed as argument to taskValue()
                                     1,            // task priority (higher values mean more process time)
                                     (xTaskHandle*) NULL                    // can return a handle to created task
                                     ),
               ec_Malloc); //}

        while (ttc_mutex_lock(ThreadInfo.Lock_Init, -1) != tqe_OK);  // wait until task has released structure
        ThreadInfo.DisplayRow++;
    }

    for (int Amount = 0; Amount < AmountConsumers; Amount++) {
        ttc_Port_t* Port = &(ThreadInfo.ConfigLED.Port);
        switch (Amount) {
#ifdef TTC_LED1
          case  0: ttc_gpio_init_variable(Port, TTC_LED1, tgm_output_push_pull); break; // loads *Port with TTC_LED1
#endif
#ifdef TTC_LED2
          case  1: ttc_gpio_init_variable(Port, TTC_LED2, tgm_output_push_pull); break; // loads *Port with TTC_LED2
#endif
          default: ttc_gpio_variable(Port, NULL, 0);                       break; // loads *Port with 0,0
        }

        Assert(0 != ttc_task_create( taskConsumer,                          //{ function to start as thread
                                     "Consumer",       // thread name (just for debugging)
                                     4 * TTC_TASK_MINIMAL_STACK_SIZE, // stack size
                                     (void *) &ThreadInfo,                  // passed as argument to taskValue()
                                     1,            // task priority (higher values mean more process time)
                                     (xTaskHandle*) NULL                    // can return a handle to created task
                                     ),
               ec_Malloc); //}
        while (ttc_mutex_lock(ThreadInfo.Lock_Init, -1) != tqe_OK);  // wait until task has released structure
        ThreadInfo.DisplayRow++;
    }
    ttc_mutex_unlock(ThreadInfo.Lock_Init);

    queuePrintJob(ThreadInfo.QueuePrint, 4, 0, "Example");
    queuePrintJob(ThreadInfo.QueuePrint, 3, 1, "Semaphores");
    queuePrintJob(ThreadInfo.QueuePrint, 0, 3, "Q=");

}
void displayQueue(ttc_SemaphoreHandle Semaphore, ttc_QueueHandle QueuePrint) {
    char QueueText[11];
    char* Writer = QueueText;
    Base_t QueueSize = ttc_queue_amount_waiting(Semaphore);
    u8_t I = 0;
    for (; (I < QueueSize/3) && (I < 10); I++)
        *Writer++ = '*';
    for (; I < 10; I++)
        *Writer++ = ' ';
    *Writer++ = 0;
    queuePrintJob(QueuePrint, 2, 3, QueueText);
}
void taskProducer(void *pvParameters) {
    ets_ThreadInfo_t* ThreadInfoPtr = (ets_ThreadInfo_t*) pvParameters;
    Assert(NULL != ThreadInfoPtr, ec_NULL);
    Assert(NULL != ThreadInfoPtr->SemaphoreProducer2Consumer, ec_InvalidArgument);
    Assert(NULL != ThreadInfoPtr->QueuePrint,                 ec_InvalidArgument);
    Assert(NULL != ThreadInfoPtr->Lock_Init,                  ec_InvalidArgument);

    ets_ThreadInfo_t ThreadInfo;
    memcpy(&ThreadInfo, pvParameters, sizeof(ets_ThreadInfo_t));
    ttc_mutex_unlock(ThreadInfo.Lock_Init); // task creator may now reuse *pvParameters

    queuePrintJob(ThreadInfo.QueuePrint, 2, ThreadInfo.DisplayRow, "P>");

    u8_t Value = 0; //(u16_t) rand() & 0xfff;
    while (1) {
        queuePrintJobV(ThreadInfo.QueuePrint, 6, ThreadInfo.DisplayRow, "+%i ", Value/10 + 1);
        for (int I = 0; I < Value; I+=10) {
            while (ttc_semaphore_give(ThreadInfo.SemaphoreProducer2Consumer) != tqe_OK);
        }

        displayQueue(ThreadInfo.SemaphoreProducer2Consumer, ThreadInfo.QueuePrint);
        mSleep(800);

        Value++;
        if (Value == 100) Value = 0;
    }
}
void taskConsumer(void *pvParameters) {
    Assert(NULL != pvParameters,                             ec_NULL);
    Assert(NULL != ((ets_ThreadInfo_t*) pvParameters)->SemaphoreProducer2Consumer, ec_InvalidArgument);
    Assert(NULL != ((ets_ThreadInfo_t*) pvParameters)->QueuePrint,                 ec_InvalidArgument);
    Assert(NULL != ((ets_ThreadInfo_t*) pvParameters)->Lock_Init,                  ec_InvalidArgument);

    ets_ThreadInfo_t ThreadInfo;
    memcpy(&ThreadInfo, pvParameters, sizeof(ets_ThreadInfo_t));
    ttc_mutex_unlock(ThreadInfo.Lock_Init); // task creator may now reuse *pvParameters

    queuePrintJob(ThreadInfo.QueuePrint, 2, ThreadInfo.DisplayRow, "C>");
    ttc_Port_t* Port_LED = &ThreadInfo.ConfigLED.Port;
    if (Port_LED->GPIOx == NULL) 
      Port_LED = NULL;              // port was not initialized: Dont use it

    u32_t Counter = 0;
    while (1) {
        if ( ttc_semaphore_take(ThreadInfo.SemaphoreProducer2Consumer, -1) == tqe_OK) {
            Counter++;
            if (Port_LED) {
              for (int I = Counter; I >= 0; I--) {
                  ttc_gpio_set_variable(Port_LED);
                  mSleep(400);
                  ttc_gpio_clr_variable(Port_LED);
                  mSleep(400);
              }
            }
#ifdef EXTENSION_400_lcd_320x240_olimex
            queuePrintJob(ThreadInfo.QueuePrint, 5, ThreadInfo.DisplayRow, "|");
            mSleep(500);
            queuePrintJob(ThreadInfo.QueuePrint, 5, ThreadInfo.DisplayRow, "-");
            mSleep(500);
            queuePrintJobV(ThreadInfo.QueuePrint, 7, ThreadInfo.DisplayRow, "%d", Counter);
            displayQueue(ThreadInfo.SemaphoreProducer2Consumer, ThreadInfo.QueuePrint);
#endif
        }
    }
}
void queuePrintJobV(ttc_QueueHandle PrintQueue, s8_t Column, s8_t Row, const char* Text, s16_t Value) {
    et_PrintJob_t PrintJob;
    PrintJob.Column = Column;
    PrintJob.Row    = Row;
    PrintJob.Value  = Value;
    strncpy(PrintJob.Text, Text, 10);

    while (ttc_queue_push_back(PrintQueue, &PrintJob, -1) != tqe_OK);
}
void queuePrintJob(ttc_QueueHandle PrintQueue, s8_t Column, s8_t Row, const char* Text) {
    queuePrintJobV(PrintQueue, Column, Row, Text, 0);
}
void taskPrint(void *pvParameters) {
    ets_ThreadInfo_t* ThreadInfoPtr = (ets_ThreadInfo_t*) pvParameters;
    Assert(NULL != ThreadInfoPtr, ec_NULL);
    Assert(NULL != ThreadInfoPtr->SemaphoreProducer2Consumer, ec_InvalidArgument);
    Assert(NULL != ThreadInfoPtr->QueuePrint,                 ec_InvalidArgument);
    Assert(NULL != ThreadInfoPtr->Lock_Init,                  ec_InvalidArgument);

    ets_ThreadInfo_t ThreadInfo;
    memcpy(&ThreadInfo, pvParameters, sizeof(ets_ThreadInfo_t));
    ttc_mutex_unlock(ThreadInfo.Lock_Init); // task creator may now reuse *pvParameters

    while (1) {
        et_PrintJob_t PrintJob;
        if ( ttc_queue_pop_front(ThreadInfo.QueuePrint, &PrintJob,  -1) == tqe_OK ) {
#ifdef EXTENSION_400_lcd_320x240_olimex
            if ( (PrintJob.Column != -1) && (PrintJob.Row != -1) )
                LCD_ConSetPos(PrintJob.Row, PrintJob.Column);
            LCD_Printf(PrintJob.Text, PrintJob.Value);
#endif
        }
    }
}

//} Function definitions

==> ToolChain_STM32_1.0.51.tar.bz: ToolChain_STM32_1.0.51/Template/examples/example_threading_semaphores.c <==
/*{ ExampleThreading::.c ************************************************
 
 Example code demonstrating how to use inter thread communication.
 
}*/

#include "example_threading_semaphores.h"

#define AmountProducers 1
#define AmountConsumers 6


#ifdef EXTENSION_500_ttc_gfx
  const char* Example    = "Example";
  const char* Threading  = "Threading";
  const char* Semaphores = "Semaphores";

  // configuration data of connected display
  ttc_gfx_generic_t* DisplayConfig = NULL;
#endif

//{ Function definitions *************************************************

void threading_init() {
#ifdef EXTENSION_500_ttc_gfx
    u8_t MaxDisplay = ttc_gfx_get_max_display_index();
    Assert(MaxDisplay > 0, ec_DeviceNotFound); // no displays defined by board makefile!
    DisplayConfig = ttc_gfx_get_configuration(1);
    Assert(DisplayConfig, ec_DeviceNotFound);

    // set color to use for first screen clear
    ttc_gfx_set_color_bg24(GFX_COLOR24_BLUE);
    ttc_gfx_set_color_fg24(GFX_COLOR24_YELLOW);
    ttc_gfx_init(1);

    // initialize gfx for use with printf()
    ttc_gfx_stdout_set(1);
    ttc_string_register_stdout(ttc_gfx_stdout_print_block, ttc_gfx_stdout_print_string);

    ttc_gfx_text_cursor_set( ttc_gfx_text_center(Example, -1), 0);
    printf(Example);
    ttc_gfx_text_cursor_set( ttc_gfx_text_center(Threading, -1), 1);
    printf(Threading);
    ttc_gfx_text_cursor_set( ttc_gfx_text_center(Semaphores, -1), 2);
    printf(Semaphores);

#endif
}
void threading_start() {
    threading_init();

    static ets_ThreadInfo_t ThreadInfo; // must be static to survive end of this function!

    // Mutex being used to wait until task has copied its ThreadInfo
    ThreadInfo.Lock_Init = ttc_mutex_create();
    Assert(NULL != ThreadInfo.Lock_Init, ec_Malloc);

    // create counting semaphore Producer -> Consumer
    ThreadInfo.SemaphoreProducer2Consumer = ttc_semaphore_create(30, 0);
    Assert(NULL != ThreadInfo.SemaphoreProducer2Consumer, ec_Malloc);

    // create queue Producer/Consumer -> Print
    ThreadInfo.QueuePrint = ttc_queue_create(5, sizeof(et_PrintJob_t));
    Assert(NULL != ThreadInfo.QueuePrint, ec_Malloc);
    
    ttc_mutex_lock(ThreadInfo.Lock_Init, 0);  // lock structure
    Assert(0 != ttc_task_create( taskPrint,                    //{ function to start as thread
                                 "Print",                      // thread name (just for debugging)
                                 TTC_TASK_MINIMAL_STACK_SIZE,  // stack size
                                 (void *) &ThreadInfo,         // passed as argument to taskValue()
                                 1,                            // task priority (higher values mean more process time)
                                 (xTaskHandle*) NULL           // can return a handle to created task
                                 ),
           ec_Malloc); //}
    while (ttc_mutex_lock(ThreadInfo.Lock_Init, -1) != tqe_OK);  // wait until task has released structure

    ThreadInfo.DisplayRow = 5;

    for (int Amount = 0; Amount < AmountProducers; Amount++) {
        Assert(0 != ttc_task_create( taskProducer,                          //{ function to start as thread
                                     "Producer",       // thread name (just for debugging)
                                     TTC_TASK_MINIMAL_STACK_SIZE, // stack size
                                     (void *) &ThreadInfo,                  // passed as argument to taskValue()
                                     1,            // task priority (higher values mean more process time)
                                     (xTaskHandle*) NULL                    // can return a handle to created task
                                     ),
               ec_Malloc); //}

        while (ttc_mutex_lock(ThreadInfo.Lock_Init, -1) != tqe_OK);  // wait until task has released structure
        ThreadInfo.DisplayRow++;
    }

    for (int Amount = 0; Amount < AmountConsumers; Amount++) {
        ttc_Port_t* Port = &(ThreadInfo.ConfigLED.Port);
        switch (Amount) {
#ifdef TTC_LED1
          case  0: ttc_gpio_init_variable(Port, TTC_LED1, tgm_output_push_pull); break; // loads *Port with TTC_LED1
#endif
#ifdef TTC_LED2
          case  1: ttc_gpio_init_variable(Port, TTC_LED2, tgm_output_push_pull); break; // loads *Port with TTC_LED2
#endif
          default: ttc_gpio_variable(Port, PIN_NULL);                       break; // loads *Port with 0,0
        }

        Assert(0 != ttc_task_create( taskConsumer,                          //{ function to start as thread
                                     "Consumer",       // thread name (just for debugging)
                                     TTC_TASK_MINIMAL_STACK_SIZE, // stack size
                                     (void *) &ThreadInfo,                  // passed as argument to taskValue()
                                     1,            // task priority (higher values mean more process time)
                                     (xTaskHandle*) NULL                    // can return a handle to created task
                                     ),
               ec_Malloc); //}
        while (ttc_mutex_lock(ThreadInfo.Lock_Init, -1) != tqe_OK);  // wait until task has released structure
        ThreadInfo.DisplayRow++;
    }
    ttc_mutex_unlock(ThreadInfo.Lock_Init);

    queuePrintJob(ThreadInfo.QueuePrint, 0, 3, "Q=");
    Base_t BytesFree = ttc_memory_get_free_size();
    queuePrintJobV(ThreadInfo.QueuePrint, 2, DisplayConfig->TextRows - 1, "free: %iB", BytesFree);

}
void displayQueue(ttc_SemaphoreHandle Semaphore, ttc_QueueHandle QueuePrint) {
    char QueueText[11];
    char* Writer = QueueText;
    Base_t QueueSize = ttc_queue_amount_waiting(Semaphore);
    u8_t I = 0;
    for (; (I < QueueSize/3) && (I < 10); I++)
        *Writer++ = '*';
    for (; I < 10; I++)
        *Writer++ = ' ';
    *Writer++ = 0;
    queuePrintJob(QueuePrint, 2, 3, QueueText);
}
void taskProducer(void *pvParameters) {
    ets_ThreadInfo_t* ThreadInfoPtr = (ets_ThreadInfo_t*) pvParameters;
    Assert(NULL != ThreadInfoPtr, ec_NULL);
    Assert(NULL != ThreadInfoPtr->SemaphoreProducer2Consumer, ec_InvalidArgument);
    Assert(NULL != ThreadInfoPtr->QueuePrint,                 ec_InvalidArgument);
    Assert(NULL != ThreadInfoPtr->Lock_Init,                  ec_InvalidArgument);

    ets_ThreadInfo_t ThreadInfo;
    memcpy(&ThreadInfo, pvParameters, sizeof(ets_ThreadInfo_t));
    ttc_mutex_unlock(ThreadInfo.Lock_Init); // task creator may now reuse *pvParameters

    queuePrintJob(ThreadInfo.QueuePrint, 2, ThreadInfo.DisplayRow, "P>");

    u8_t Value = 0; //(u16_t) rand() & 0xfff;
    while (1) {
        queuePrintJobV(ThreadInfo.QueuePrint, 6, ThreadInfo.DisplayRow, "+%i ", Value/10 + 1);
        for (int I = 0; I < Value; I+=10) {
            while (ttc_semaphore_give(ThreadInfo.SemaphoreProducer2Consumer) != tqe_OK);
        }

        displayQueue(ThreadInfo.SemaphoreProducer2Consumer, ThreadInfo.QueuePrint);
        mSleep(800);

        Value++;
        if (Value == 100) Value = 0;
    }
}
void taskConsumer(void *pvParameters) {
    Assert(NULL != pvParameters,                             ec_NULL);
    Assert(NULL != ((ets_ThreadInfo_t*) pvParameters)->SemaphoreProducer2Consumer, ec_InvalidArgument);
    Assert(NULL != ((ets_ThreadInfo_t*) pvParameters)->QueuePrint,                 ec_InvalidArgument);
    Assert(NULL != ((ets_ThreadInfo_t*) pvParameters)->Lock_Init,                  ec_InvalidArgument);

    ets_ThreadInfo_t ThreadInfo;
    memcpy(&ThreadInfo, pvParameters, sizeof(ets_ThreadInfo_t));
    ttc_mutex_unlock(ThreadInfo.Lock_Init); // task creator may now reuse *pvParameters

    queuePrintJob(ThreadInfo.QueuePrint, 2, ThreadInfo.DisplayRow, "C>");
    ttc_Port_t* Port_LED = &ThreadInfo.ConfigLED.Port;
    if (Port_LED->GPIOx == NULL) 
      Port_LED = NULL;              // port was not initialized: Dont use it

    u32_t Counter = 0;
    while (1) {
        if ( ttc_semaphore_take(ThreadInfo.SemaphoreProducer2Consumer, -1) == tqe_OK) {
            Counter++;
            if (Port_LED) {
              for (int I = Counter; I >= 0; I--) {
                  ttc_gpio_set_variable(Port_LED);
                  mSleep(400);
                  ttc_gpio_clr_variable(Port_LED);
                  mSleep(400);
              }
            }
#ifdef EXTENSION_500_ttc_gfx
            queuePrintJob(ThreadInfo.QueuePrint, 5, ThreadInfo.DisplayRow, "|");
            mSleep(500);
            queuePrintJob(ThreadInfo.QueuePrint, 5, ThreadInfo.DisplayRow, "-");
            mSleep(500);
            queuePrintJobV(ThreadInfo.QueuePrint, 7, ThreadInfo.DisplayRow, "%d", Counter);
            displayQueue(ThreadInfo.SemaphoreProducer2Consumer, ThreadInfo.QueuePrint);
#endif
        }
    }
}
void queuePrintJobV(ttc_QueueHandle PrintQueue, s8_t Column, s8_t Row, const char* Text, s32_t Value) {
    et_PrintJob_t PrintJob;
    PrintJob.Column = Column;
    PrintJob.Row    = Row;
    PrintJob.Value  = Value;
    strncpy(PrintJob.Text, Text, 10);

    while (ttc_queue_push_back(PrintQueue, &PrintJob, -1) != tqe_OK);
}
void queuePrintJob(ttc_QueueHandle PrintQueue, s8_t Column, s8_t Row, const char* Text) {
    queuePrintJobV(PrintQueue, Column, Row, Text, 0);
}
void taskPrint(void *pvParameters) {
    ets_ThreadInfo_t* ThreadInfoPtr = (ets_ThreadInfo_t*) pvParameters;
    Assert(NULL != ThreadInfoPtr, ec_NULL);
    Assert(NULL != ThreadInfoPtr->SemaphoreProducer2Consumer, ec_InvalidArgument);
    Assert(NULL != ThreadInfoPtr->QueuePrint,                 ec_InvalidArgument);
    Assert(NULL != ThreadInfoPtr->Lock_Init,                  ec_InvalidArgument);

    ets_ThreadInfo_t ThreadInfo;
    memcpy(&ThreadInfo, pvParameters, sizeof(ets_ThreadInfo_t));
    ttc_mutex_unlock(ThreadInfo.Lock_Init); // task creator may now reuse *pvParameters

    while (1) {
        et_PrintJob_t PrintJob;
        if ( ttc_queue_pop_front(ThreadInfo.QueuePrint, &PrintJob,  -1) == tqe_OK ) {
#ifdef EXTENSION_500_ttc_gfx
            if ( (PrintJob.Column != -1) && (PrintJob.Row != -1) )
                ttc_gfx_text_cursor_set(PrintJob.Column, PrintJob.Row);
            printf(PrintJob.Text, PrintJob.Value);
#endif
        }
    }
}

//} Function definitions
